// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with the Dialogic runtime libraries to produce a executable image
// without requiring Dialogic's sources to be supplied so long as each
// source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#define	USE_OLD_CONSTS	1

#include <ivrconfig.h>
#include <server.h>
#include <cc++/audio.h>

#include <srllib.h>
#include <dxxxlib.h>
#include <dtilib.h>
#include <sctools.h>

extern "C" {
	int uio_read(int fd, char *buffer, unsigned len);
	int uio_write(int fd, char *buffer, unsigned len);
	long uio_seek(int fd, long offset, int mode);
};

typedef enum
{
	NT_ANALOG = CT_NTANALOG,
	NT_T1 = CT_NTT1,
	NT_E1 = CT_NTE1,
	NT_ISDN
} interface_t;

typedef	struct
{
	interface_t	interface;
	int chdev;
	int tsdev;
	char gcname[65];
}	devices_t;

class DialogicTrunk;

typedef	bool (DialogicTrunk::*trunkhandler_t)(TrunkEvent *event);

#define	HOLD(msg)	dialogicivr.EnterMutex();
#define	RELEASE		dialogicivr.LeaveMutex();
#define	WAIT		dialogicivr.EnterMutex(); sr_waitevt(0);	
#define	CONTINUE	dialogicivr.LeaveMutex(); continue;

class DialogicTrunk : private TimerPort, private Trunk, private Thread, private AudioFile, private AudioService
{
private:
	friend class DialogicDriver;
	static char status[20 * 30];

	DialogicTrunk *join;
	interface_t interface;
	trunkhandler_t handler;
	time_t lastring;
	bool routing;
	int chdev, tsdev;
	int iocnt;
	DV_DIGIT digbuf[33];
	DV_TPT digtpt[4];
	DX_IOTT iofile, iotone;
	int evbuf[2];

	bool Join(DialogicTrunk *trunk);
	void Part(void);
	void tsConnect(void);
	void tsRelease(void);

	bool postEvent(TrunkEvent *event);
	void putEvent(TrunkEvent *evt);
	void Run(void);
	void Exit(void);
	void TrunkStep(trunkstep_t step);
	void getName(char *buffer);
	void setDTMFDetect(bool flag);
	unsigned long getIdleTime(void);
	DV_TPT *getPending(timeout_t delay);
	void getDigits(timeout_t delay);
	void setHook(bool hook);
	void setRing(int rings);
	void stopChannel(void);
	void playAudio(void);
	void recordAudio(void);
	void playTone(phTone *tone);
	int getCallerId(int mode, unsigned char *cid);
	char *getContinuation(void);

	bool answerHandler(TrunkEvent *event);
	bool stepHandler(TrunkEvent *event);
	bool idleHandler(TrunkEvent *event);
	bool dialHandler(TrunkEvent *event);
	bool sleepHandler(TrunkEvent *event);
	bool collectHandler(TrunkEvent *event);
	bool ringHandler(TrunkEvent *event);
	bool hangupHandler(TrunkEvent *event);
	bool playHandler(TrunkEvent *event);
	bool recordHandler(TrunkEvent *event);
	bool joinHandler(TrunkEvent *event);
	bool flashonHandler(TrunkEvent *event);
	bool flashoffHandler(TrunkEvent *event);

	bool scrJoin(void);
	bool scrWait(void);

	void endRecord(void);
	void endPlay(void);

	DialogicTrunk(int id, int ch, int ts);
	~DialogicTrunk();
	
	friend int uio_read(int fd, char *buffer, unsigned len);
	friend int uio_write(int fd, char *buffer, unsigned len);
	friend long uio_seek(int fd, long offset, int mode);	
};

class DialogicConfig : public Keydata
{
public:
	DialogicConfig();

	bool isAnalog(int card = 0);
	bool getRouting();

	inline const char *getDebounce(void)
		{return getLast("debounce");};

	inline const char *getTalkback(void)
		{return getLast("talkback");};

	inline int getInterval(void)
		{return atoi(getLast("interval"));};

	inline int getBuffers(void)
		{return atoi(getLast("buffers"));};

	inline int getVoice(void)
		{return atoi(getLast("voice"));};

	inline int getDigital(void)
		{return atoi(getLast("digital"));};

	inline int getTimeslots(void)
		{return atoi(getLast("timeslots"));};

	inline const char *getAnalog(void)
		{return getLast("analog");};

	inline const char *getProtocol(void)
		{return getLast("protocol");};
};

class DialogicDriver : public Driver, public DialogicConfig, public Thread
{
private:
	DialogicTrunk **ports;
	DialogicTrunk **maps;
	devices_t *devmap;
	int mode;
	int port_count, port_used, level;
	volatile bool running;
	DX_UIO uio;
	DV_TPT tpt;

	void Run(void);

	friend int uio_read(int fd, char *buffer, unsigned len);
	friend int uio_write(int fd, char *buffer, unsigned len);
	friend long uio_seek(int fd, long offset, int mode);	
	
public:
	DialogicDriver();
	~DialogicDriver();

	void Hold(const char *str);
	void Release(void);

	int Start(void);
	void Stop(void);

	int getTrunkCount(void)
		{return port_count;};

	Trunk *getTrunkPort(int id);
	aaScript *getScript(void);
	DV_TPT *getTPT(void)
		{return &tpt;};
};


extern DialogicDriver dialogicivr;
